#pragma once

#ifdef __cplusplus
extern "C" {
#endif


char* mem_logger_read_line();
void mem_logger_print(const int level, const char * format, ... );
void use_mem_logger();


#ifdef __cplusplus
}
#endif
