// Autogenerated do not edit
#ifndef __SIM_CORE_CONFIG_H__
#define __SIM_CORE_CONFIG_H__

#define CONFIG_SIMAVR_VERSION "v1.0"
#define CONFIG_90USB162 1
#define CONFIG_MEGA1280 1
#define CONFIG_MEGA1281 1
#define CONFIG_MEGA128 1
#define CONFIG_MEGA164 1
#define CONFIG_MEGA168 1
#define CONFIG_MEGA16M1 1
#define CONFIG_MEGA324 1
#define CONFIG_MEGA328 1
#define CONFIG_MEGA48 1
#define CONFIG_MEGA644 1
#define CONFIG_MEGA88 1
#define CONFIG_MEGA8 1
#define CONFIG_MEGAX4 1
#define CONFIG_MEGAX8 1
#define CONFIG_MEGAX 1
#define CONFIG_MEGAXM1 1
#define CONFIG_TINY13 1
#define CONFIG_TINY2313 1
#define CONFIG_TINY24 1
#define CONFIG_TINY25 1
#define CONFIG_TINY44 1
#define CONFIG_TINY45 1
#define CONFIG_TINY84 1
#define CONFIG_TINY85 1
#define CONFIG_TINYX4 1
#define CONFIG_TINYX5 1

#endif
